@@ -1,13 +1,17 @@
 /*
  * The implementation of the different descriptors.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
+ * This copy of SIP may also used under the terms of the GNU General Public
+ * License v2 or v3 as published by the Free Software Foundation which can be
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -21,7 +25,8 @@
 
 /*****************************************************************************
  * A method descriptor.  We don't use the similar Python descriptor because it
- * doesn't support a method having static and non-static overloads.
+ * doesn't support a method having static and non-static overloads, and we
+ * handle mixins via a delegate.
  *****************************************************************************/
 
 
@@ -29,6 +34,9 @@
 static PyObject *sipMethodDescr_descr_get(PyObject *self, PyObject *obj,
         PyObject *type);
 static PyObject *sipMethodDescr_repr(PyObject *self);
+static int sipMethodDescr_traverse(PyObject *self, visitproc visit, void *arg);
+static int sipMethodDescr_clear(PyObject *self);
+static void sipMethodDescr_dealloc(PyObject *self);
 
 
 /*
@@ -39,6 +47,9 @@ typedef struct _sipMethodDescr {
 
     /* The method definition. */
     PyMethodDef *pmd;
+
+    /* The mixin name, if any. */
+    PyObject *mixin_name;
 } sipMethodDescr;
 
 
@@ -50,7 +61,7 @@ PyTypeObject sipMethodDescr_Type = {
     "sip.methoddescriptor", /* tp_name */
     sizeof (sipMethodDescr),    /* tp_basicsize */
     0,                      /* tp_itemsize */
-    0,                      /* tp_dealloc */
+    sipMethodDescr_dealloc, /* tp_dealloc */
     0,                      /* tp_print */
     0,                      /* tp_getattr */
     0,                      /* tp_setattr */
@@ -65,10 +76,10 @@ PyTypeObject sipMethodDescr_Type = {
     0,                      /* tp_getattro */
     0,                      /* tp_setattro */
     0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,  /* tp_flags */
     0,                      /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
+    sipMethodDescr_traverse,/* tp_traverse */
+    sipMethodDescr_clear,   /* tp_clear */
     0,                      /* tp_richcompare */
     0,                      /* tp_weaklistoffset */
     0,                      /* tp_iter */
@@ -79,6 +90,23 @@ PyTypeObject sipMethodDescr_Type = {
     0,                      /* tp_base */
     0,                      /* tp_dict */
     sipMethodDescr_descr_get,   /* tp_descr_get */
+    0,                      /* tp_descr_set */
+    0,                      /* tp_dictoffset */
+    0,                      /* tp_init */
+    0,                      /* tp_alloc */
+    0,                      /* tp_new */
+    0,                      /* tp_free */
+    0,                      /* tp_is_gc */
+    0,                      /* tp_bases */
+    0,                      /* tp_mro */
+    0,                      /* tp_cache */
+    0,                      /* tp_subclasses */
+    0,                      /* tp_weaklist */
+    0,                      /* tp_del */
+    0,                      /* tp_version_tag */
+#if PY_VERSION_HEX >= 0x03040000
+    0,                      /* tp_finalize */
+#endif
 };
 
 
@@ -90,7 +118,28 @@ PyObject *sipMethodDescr_New(PyMethodDef *pmd)
     PyObject *descr = PyType_GenericAlloc(&sipMethodDescr_Type, 0);
 
     if (descr != NULL)
+    {
         ((sipMethodDescr *)descr)->pmd = pmd;
+        ((sipMethodDescr *)descr)->mixin_name = NULL;
+    }
+
+    return descr;
+}
+
+
+/*
+ * Return a new method descriptor based on an existing one and a mixin name.
+ */
+PyObject *sipMethodDescr_Copy(PyObject *orig, PyObject *mixin_name)
+{
+    PyObject *descr = PyType_GenericAlloc(&sipMethodDescr_Type, 0);
+
+    if (descr != NULL)
+    {
+        ((sipMethodDescr *)descr)->pmd = ((sipMethodDescr *)orig)->pmd;
+        ((sipMethodDescr *)descr)->mixin_name = mixin_name;
+        Py_INCREF(mixin_name);
+    }
 
     return descr;
 }
@@ -104,8 +153,12 @@ static PyObject *sipMethodDescr_descr_get(PyObject *self, PyObject *obj,
 {
     sipMethodDescr *md = (sipMethodDescr *)self;
 
+    (void)type;
+
     if (obj == Py_None)
         obj = NULL;
+    else if (md->mixin_name != NULL)
+        obj = PyObject_GetAttr(obj, md->mixin_name);
 
     return PyCFunction_New(md->pmd, obj);
 }
@@ -129,6 +182,47 @@ static PyObject *sipMethodDescr_repr(PyObject *self)
 }
 
 
+/*
+ * The descriptor's traverse slot.
+ */
+static int sipMethodDescr_traverse(PyObject *self, visitproc visit, void *arg)
+{
+    if (((sipMethodDescr *)self)->mixin_name != NULL)
+    {
+        int vret = visit(((sipMethodDescr *)self)->mixin_name, arg);
+
+        if (vret != 0)
+            return vret;
+    }
+
+    return 0;
+}
+
+
+/*
+ * The descriptor's clear slot.
+ */
+static int sipMethodDescr_clear(PyObject *self)
+{
+    PyObject *tmp = ((sipMethodDescr *)self)->mixin_name;
+
+    ((sipMethodDescr *)self)->mixin_name = NULL;
+    Py_XDECREF(tmp);
+
+    return 0;
+}
+
+
+/*
+ * The descriptor's dealloc slot.
+ */
+static void sipMethodDescr_dealloc(PyObject *self)
+{
+    sipMethodDescr_clear(self);
+    Py_TYPE(self)->tp_free(self);
+}
+
+
 /*****************************************************************************
  * A variable descriptor.  We don't use the similar Python descriptor because
  * it doesn't support static variables.
@@ -140,6 +234,10 @@ static PyObject *sipVariableDescr_descr_get(PyObject *self, PyObject *obj,
         PyObject *type);
 static int sipVariableDescr_descr_set(PyObject *self, PyObject *obj,
         PyObject *value);
+static int sipVariableDescr_traverse(PyObject *self, visitproc visit,
+        void *arg);
+static int sipVariableDescr_clear(PyObject *self);
+static void sipVariableDescr_dealloc(PyObject *self);
 
 
 /*
@@ -152,7 +250,13 @@ typedef struct _sipVariableDescr {
     sipVariableDef *vd;
 
     /* The generated type definition. */
-    const sipClassTypeDef *ctd;
+    const sipTypeDef *td;
+
+    /* The generated container definition. */
+    const sipContainerDef *cod;
+
+    /* The mixin name, if any. */
+    PyObject *mixin_name;
 } sipVariableDescr;
 
 
@@ -164,7 +268,7 @@ PyTypeObject sipVariableDescr_Type = {
     "sip.variabledescriptor",   /* tp_name */
     sizeof (sipVariableDescr),  /* tp_basicsize */
     0,                      /* tp_itemsize */
-    0,                      /* tp_dealloc */
+    sipVariableDescr_dealloc,   /* tp_dealloc */
     0,                      /* tp_print */
     0,                      /* tp_getattr */
     0,                      /* tp_setattr */
@@ -179,10 +283,10 @@ PyTypeObject sipVariableDescr_Type = {
     0,                      /* tp_getattro */
     0,                      /* tp_setattro */
     0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_GC,  /* tp_flags */
     0,                      /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
+    sipVariableDescr_traverse,  /* tp_traverse */
+    sipVariableDescr_clear, /* tp_clear */
     0,                      /* tp_richcompare */
     0,                      /* tp_weaklistoffset */
     0,                      /* tp_iter */
@@ -194,6 +298,24 @@ PyTypeObject sipVariableDescr_Type = {
     0,                      /* tp_dict */
     sipVariableDescr_descr_get, /* tp_descr_get */
     sipVariableDescr_descr_set, /* tp_descr_set */
+    0,                      /* tp_dictoffset */
+    0,                      /* tp_init */
+    0,                      /* tp_alloc */
+    0,                      /* tp_new */
+    0,                      /* tp_free */
+    0,                      /* tp_is_gc */
+    0,                      /* tp_bases */
+    0,                      /* tp_mro */
+    0,                      /* tp_cache */
+    0,                      /* tp_subclasses */
+    0,                      /* tp_weaklist */
+    0,                      /* tp_del */
+#if PY_VERSION_HEX >= 0x02060000
+    0,                      /* tp_version_tag */
+#endif
+#if PY_VERSION_HEX >= 0x03040000
+    0,                      /* tp_finalize */
+#endif
 };
 
 
@@ -205,14 +327,37 @@ static int get_instance_address(sipVariableDescr *vd, PyObject *obj,
 /*
  * Return a new method descriptor for the given getter/setter.
  */
-PyObject *sipVariableDescr_New(sipVariableDef *vd, const sipClassTypeDef *ctd)
+PyObject *sipVariableDescr_New(sipVariableDef *vd, const sipTypeDef *td,
+        const sipContainerDef *cod)
 {
     PyObject *descr = PyType_GenericAlloc(&sipVariableDescr_Type, 0);
 
     if (descr != NULL)
     {
         ((sipVariableDescr *)descr)->vd = vd;
-        ((sipVariableDescr *)descr)->ctd = ctd;
+        ((sipVariableDescr *)descr)->td = td;
+        ((sipVariableDescr *)descr)->cod = cod;
+        ((sipVariableDescr *)descr)->mixin_name = NULL;
+    }
+
+    return descr;
+}
+
+
+/*
+ * Return a new variable descriptor based on an existing one and a mixin name.
+ */
+PyObject *sipVariableDescr_Copy(PyObject *orig, PyObject *mixin_name)
+{
+    PyObject *descr = PyType_GenericAlloc(&sipVariableDescr_Type, 0);
+
+    if (descr != NULL)
+    {
+        ((sipVariableDescr *)descr)->vd = ((sipVariableDescr *)orig)->vd;
+        ((sipVariableDescr *)descr)->td = ((sipVariableDescr *)orig)->td;
+        ((sipVariableDescr *)descr)->cod = ((sipVariableDescr *)orig)->cod;
+        ((sipVariableDescr *)descr)->mixin_name = mixin_name;
+        Py_INCREF(mixin_name);
     }
 
     return descr;
@@ -231,7 +376,7 @@ static PyObject *sipVariableDescr_descr_get(PyObject *self, PyObject *obj,
     if (get_instance_address(vd, obj, &addr) < 0)
         return NULL;
 
-    return vd->vd->vd_getter(addr, type);
+    return ((sipVariableGetterFunc)vd->vd->vd_getter)(addr, obj, type);
 }
 
 
@@ -249,7 +394,7 @@ static int sipVariableDescr_descr_set(PyObject *self, PyObject *obj,
     {
         PyErr_Format(PyExc_AttributeError,
                 "'%s' object attribute '%s' is read-only",
-                sipPyNameOfClass(vd->ctd), vd->vd->vd_name);
+                sipPyNameOfContainer(vd->cod, vd->td), vd->vd->vd_name);
 
         return -1;
     }
@@ -257,7 +402,7 @@ static int sipVariableDescr_descr_set(PyObject *self, PyObject *obj,
     if (get_instance_address(vd, obj, &addr) < 0)
         return -1;
 
-    return vd->vd->vd_setter(addr, value, obj);
+    return ((sipVariableSetterFunc)vd->vd->vd_setter)(addr, value, obj);
 }
 
 
@@ -269,7 +414,7 @@ static int get_instance_address(sipVariableDescr *vd, PyObject *obj,
 {
     void *addr;
 
-    if (vd->vd->vd_is_static)
+    if (vd->vd->vd_type == ClassVariable)
     {
         addr = NULL;
     }
@@ -280,13 +425,16 @@ static int get_instance_address(sipVariableDescr *vd, PyObject *obj,
         {
             PyErr_Format(PyExc_AttributeError,
                     "'%s' object attribute '%s' is an instance attribute",
-                    sipPyNameOfClass(vd->ctd), vd->vd->vd_name);
+                    sipPyNameOfContainer(vd->cod, vd->td), vd->vd->vd_name);
 
             return -1;
         }
 
+        if (vd->mixin_name != NULL)
+            obj = PyObject_GetAttr(obj, vd->mixin_name);
+
         /* Get the C++ instance. */
-        if ((addr = sip_api_get_cpp_ptr((sipSimpleWrapper *)obj, &vd->ctd->ctd_base)) == NULL)
+        if ((addr = sip_api_get_cpp_ptr((sipSimpleWrapper *)obj, vd->td)) == NULL)
             return -1;
     }
 
@@ -294,3 +442,44 @@ static int get_instance_address(sipVariableDescr *vd, PyObject *obj,
 
     return 0;
 }
+
+
+/*
+ * The descriptor's traverse slot.
+ */
+static int sipVariableDescr_traverse(PyObject *self, visitproc visit, void *arg)
+{
+    if (((sipVariableDescr *)self)->mixin_name != NULL)
+    {
+        int vret = visit(((sipVariableDescr *)self)->mixin_name, arg);
+
+        if (vret != 0)
+            return vret;
+    }
+
+    return 0;
+}
+
+
+/*
+ * The descriptor's clear slot.
+ */
+static int sipVariableDescr_clear(PyObject *self)
+{
+    PyObject *tmp = ((sipVariableDescr *)self)->mixin_name;
+
+    ((sipVariableDescr *)self)->mixin_name = NULL;
+    Py_XDECREF(tmp);
+
+    return 0;
+}
+
+
+/*
+ * The descriptor's dealloc slot.
+ */
+static void sipVariableDescr_dealloc(PyObject *self)
+{
+    sipVariableDescr_clear(self);
+    Py_TYPE(self)->tp_free(self);
+}
